import urllib.request
import json
from yt_concate.settings import API_KEY
from yt_concate.pipeline.steps.step import Step
import logging


class GetVideoLinks(Step):

    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        channel_id = inputs["channel_id"]
        if utils.check_video_list_exists(channel_id):
            video_links = self.read_file(utils, channel_id)
            logger.warning('Video list already existed.')
            return video_links



        base_video_url = 'https://www.youtube.com/watch?v='
        base_search_url = 'https://www.googleapis.com/youtube/v3/search?'

        first_url = base_search_url+'key={}&channelId={}&part=snippet,id&order=date&maxResults=25'.format(API_KEY, channel_id)

        video_links = []
        url = first_url
        while True:
            inp = urllib.request.urlopen(url)
            resp = json.load(inp)

            for i in resp['items']:
                if i['id']['kind'] == "youtube#video":
                    video_links.append(base_video_url + i['id']['videoId'])

            try:
                next_page_token = resp['nextPageToken']
                url = first_url + '&pageToken={}'.format(next_page_token)
            except:
                break

        self.write_to_file(video_links, utils, channel_id)
        return video_links

    @staticmethod
    def write_to_file(video_list, utils, channel_id):
        path = utils.get_video_list_path(channel_id)
        with open(path, 'w', encoding='utf-8') as fp:
            for url in video_list:
                fp.write(url+'\n')


    def read_file(self, utils, channel_id):
        video_links = []
        path = utils.get_video_list_path(channel_id)
        with open(path,'r') as fp:
            for url in fp:
                video_links.append(url.rstrip('\n'))
        return video_links





class Found:
    def __init__(self, yt, caption, caption_time):
        self.yt = yt
        self.caption = caption
        self.caption_time = caption_time

    def __repr__(self):
        return f'<Found object -> video id:{self.yt.id},caption:{self.caption},time:{self.caption_time}'



from .step import Step
from yt_concate.model.found import Found
import logging

class Search(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        logger.info('Search for corresponded search term...')
        search_term = inputs["search_term"]
        founds = []
        for yt in data:
            caption = yt.caption
            if caption:
                for caption_key in caption:
                    if search_term in caption_key:
                        founds.append(Found(yt, caption_key, caption[caption_key]))
        return founds










import sys
sys.path.append('../')
import getopt
from yt_concate.pipeline.pipeline import Pipeline
from yt_concate.pipeline.steps.preflight import Preflight
from yt_concate.pipeline.steps.get_video_links import GetVideoLinks
from yt_concate.pipeline.steps.initialize_yt import InitializeYt
from yt_concate.pipeline.steps.get_video_captions import GetVideoCaption
from yt_concate.pipeline.steps.search_term import Search
from yt_concate.pipeline.steps.download_video import DownloadVideo
from yt_concate.pipeline.steps.edit_video import EditVideo
from yt_concate.pipeline.steps.postflight import Postflight
from yt_concate.utils import Utils
from distutils.util import strtobool
from yt_concate.log import set_logger

#"UCV0qA-eDDICsRR9rPcnG7tw"


def print_usage():
    print('Options:')
    print('{:<6s} {:<12s}  channel id of searching youtube video(must input)'.format('-c','--channelid'))
    print('{:<6s} {:<12s}  search term of youtube video(must input)'.format('-s', '--searchterm'))
    print('{:<6s} {:<12s}  limit of clips to be edited into final video'.format('-l', '--limit'))
    print('{:<6s} {:<12s}  remove downloading files produced in program process'.format('', '--cleanup'))

def main():
    inputs = {
        "channel_id": None,
        "search_term": None,
        "limit": 20,
        "cleanup": False,

    }

    steps = [
        Preflight(),
        GetVideoLinks(),
        InitializeYt(),
        GetVideoCaption(),
        Search(),
        DownloadVideo(),
        EditVideo(),
        Postflight(),
    ]

    shortopt = 'hc:s:l:'
    longopt = 'channelid= searchterm= limit= cleanup='.split()

    try:
        opts, args = getopt.getopt(sys.argv[1:], shortopts=shortopt, longopts=longopt )
    except getopt.GetoptError:
        print_usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_usage()
            sys.exit(0)
        elif opt in ("-c", "--channelid"):
            inputs['channel_id'] = arg
        elif opt in ("-s", "--searchterm"):
            inputs['search_term'] = arg
        elif opt in ("-l", "--limit"):
            inputs['limit'] = arg
        elif opt == "--cleanup":
            inputs['cleanup'] = bool(strtobool(arg))

    if not inputs['channel_id'] or not inputs['search_term']:
        print('Must input channelid and searchterm.')
        print_usage()
        sys.exit(0)
    try:
        inputs['limit'] = int(inputs['limit'])
    except:
        print('Please input integer value for limit.')
        print_usage()
        sys.exit(0)

    set_logger()
    utils = Utils()
    factory = Pipeline(steps)
    factory.run(inputs, utils)


if __name__ == '__main__':
    main()













class Pipeline:

    def __init__(self, steps):
        self.steps = steps

    def run(self, inputs, utils):
        data = None
        for step in self.steps:
            try:
                data = step.process(inputs, utils, data)
            except:
                print('Error happening in step process.')















import time
import concurrent.futures
from pytube import YouTube
from .step import Step
from yt_concate.settings import VIDEO_DIR
import logging

class DownloadVideo(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        start = time.perf_counter()
        yt_set = set([found.yt for found in data])
        yts = []
        for yt in list(yt_set):
            path = yt.get_video_filepath()
            if utils.check_video_file_exist(path):
                logger.warning(f'Video for : {yt.id} existed.')
                continue
            yts.append(yt)

        with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
            executor.map(self.download_single_video, yts)

        end = time.perf_counter()
        logger.info(f'Video downloading elapsed time:{round(end - start, 2)} secs.')
        return data


    def download_single_video(self, yt):
        logger = logging.getLogger('record')
        try:
            logger.info(f'Downloading video : {yt.id}')
            name = f'{yt.id}.mp4'
            video = YouTube(yt.url)
            video.streams.first().download(VIDEO_DIR, filename=name)
        except:
            logger.warning(f'Video downloading error : {yt.id}')




import logging
from yt_concate.pipeline.steps.step import Step



class Preflight(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        logger.info('In preflight.')
        utils.create_downloads_dirs()



from .step import Step
from yt_concate.model.yt import Yt
import logging

class InitializeYt(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        logger.warning('creating YT objects')
        yts = [Yt(url) for url in data]
        return yts











import os
from dotenv import load_dotenv

load_dotenv()
API_KEY = os.getenv("API_KEY")

DOWNLOAD_DIR = "downloads"
VIDEO_DIR = os.path.join(DOWNLOAD_DIR,"videos")
CAPTION_DIR = os.path.join(DOWNLOAD_DIR,"captions")
OUTPUT_DIR = 'output'








import logging


def set_logger():
    logger = logging.getLogger('record')
    logger.setLevel('DEBUG')
    formatter = logging.Formatter('%(levelname)s : %(message)s : %(asctime)s')
    file_handler = logging.FileHandler('record.log')
    file_handler.setLevel('DEBUG')
    file_handler.setFormatter(formatter)
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel('WARNING')
    stream_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    logger.addHandler(stream_handler)






import time
import concurrent.futures
import json
from youtube_transcript_api import YouTubeTranscriptApi
from .step import Step
import logging

class GetVideoCaption(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        start = time.perf_counter()
        caption_pool = []
        for yt in data:
            path = yt.get_caption_filepath()
            if utils.check_caption_file_exist(path):
                logger.warning('Caption file existed for '+yt.id)
                caption = self.read_file(path)
                caption_after = self.reformat(caption)
                yt.caption = caption_after
                continue
            caption_pool.append(yt)

        with concurrent.futures.ThreadPoolExecutor(max_workers=30) as executor:
            executor.map(self.get_single_caption, caption_pool)

        end = time.perf_counter()
        logger.info(f'Caption downloading elapsed time:{round(end-start,2)} secs.')

        return data


    def write_to_file(self, caption, yt):
        path = yt.get_caption_filepath()
        with open(path, 'w', encoding='utf-8') as fp:
            for line in caption:
                fp.write(json.dumps(line)+'\n')

    def read_file(self,path):
        caption = []
        with open(path, 'r') as fp:
            for line in fp:
                caption.append(json.loads(line.rstrip('\n')))
        return caption

    def get_single_caption(self, yt):
        logger = logging.getLogger('record')
        try:
            logger.info('Downloading captions: '+ yt.id)
            caption = YouTubeTranscriptApi.get_transcript(yt.id)
            self.write_to_file(caption, yt)
            caption_after = self.reformat(caption)
            yt.caption = caption_after
        except:
            logger.warning('Caption is disabled for'+ yt.id)

    def reformat(self, caption):
        caption_after = {}
        for line in caption:
            caption_after[line["text"]] = [line["start"], line["duration"]]
        return caption_after






















import os
from .settings import DOWNLOAD_DIR
from .settings import VIDEO_DIR
from .settings import CAPTION_DIR
from .settings import OUTPUT_DIR


class Utils:

    @staticmethod
    def create_downloads_dirs():
        os.makedirs(DOWNLOAD_DIR, exist_ok=True)
        os.makedirs(VIDEO_DIR, exist_ok=True)
        os.makedirs(CAPTION_DIR, exist_ok=True)
        os.makedirs(OUTPUT_DIR, exist_ok=True)

    def get_video_list_path(self, channel_id):
        return os.path.join(DOWNLOAD_DIR, channel_id+'.txt')


    def check_video_list_exists(self, channel_id):
        path = self.get_video_list_path(channel_id)
        return os.path.exists(path) and os.path.getsize(path) > 0

    def check_caption_file_exist(self,path):
        return os.path.exists(path) and os.path.getsize(path) > 0

    def check_video_file_exist(self,path):
        return os.path.exists(path) and os.path.getsize(path) > 0









import os
import shutil
from .step import Step
from yt_concate.settings import CAPTION_DIR
from yt_concate.settings import VIDEO_DIR
import logging


class Postflight(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        if inputs['cleanup']:
            logger.info('In postflight')
            os.remove(utils.get_video_list_path(inputs["channel_id"]))
            shutil.rmtree(CAPTION_DIR)
            shutil.rmtree(VIDEO_DIR)
            logger.warning('All related downloading file has been removed.')



import os
from yt_concate.settings import CAPTION_DIR
from yt_concate.settings import VIDEO_DIR


class Yt:
    def __init__(self, url):
        self.url = url
        self.id = self.get_video_id(self.url)
        self.caption = None

    def __str__(self):
        return f'<Video id:{self.id} , caption:{self.caption}>'

    def __repr__(self):
        return f'<Yt object -> id:{self.id} , caption:{self.caption}>'


    def get_video_id(self, url):
        return url.split("v=")[1]

    def get_caption_filepath(self):
        return os.path.join(CAPTION_DIR, self.id+'.txt')

    def get_video_filepath(self):
        return os.path.join(VIDEO_DIR, self.id+'.mp4')

















import os
from moviepy.editor import VideoFileClip
from moviepy.editor import concatenate_videoclips
from .step import Step
from yt_concate.settings import OUTPUT_DIR
import logging


class EditVideo(Step):
    def process(self, inputs, utils, data):
        logger = logging.getLogger('record')
        logger.warning('Editing final video...')
        clips = []
        if inputs['limit'] > len(data):
            limit = len(data)
        else:
            limit = inputs['limit']
        for found in data[:limit]:
            s, e = self.parse_clip_time(found)
            try:
                clips.append(VideoFileClip(found.yt.get_video_filepath()).subclip(s, e))
            except:
                logger.error('Error happened while clip video:'+ found.yt.id)
        final_clip = concatenate_videoclips(clips)
        final_clip.write_videofile(os.path.join(OUTPUT_DIR, inputs["channel_id"]+'.mp4'))
        return


    def parse_clip_time(self,found):
        start = found.caption_time[0]
        end = found.caption_time[0] + found.caption_time[1]
        return start, end
